<?php

/**
 * Clase que interactua y realiza las operaciones relacionadas a la base de datos
 *
 * @author Eliud Trejo
 */

require_once FS_CLASSES . 'Plantillas.php';
class PlantillasDAO extends Plantillas {
	
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con la lista de todos elementos
	* @param $id id en caso de requerir solo un registro en especifico.
	* @return array
	*/	
	public function listarPlantillas( $id = null ){
		$data['id'] = $id;
		return $this->executeSP('stp_listarPlantillas(:id)', $data);
	}
	
	public function plantillasUsuario( $id_usuario = null, $id_tipo_comprobante){
		$data = array(
			'id_usuario' => $id_usuario,
			'id_tipo_comprobante' => $id_tipo_comprobante
		);
		
		return $this->executeSP('stp_plantillasUsuario(:id_usuario, :id_tipo_comprobante)', $data);
	}
	
	/**
	 * Obtiene la plantilla configurada por tipo de comprobante para una cuenta de usuario
	 * @param $id_usuario int ID de la cuenta de usuairo
	 * @param $id_tipo_comprobante int ID del tipo de comprobante
	 * @return array
	 */
	public function plantillaComprobante($id_usuario, $id_tipo_comprobante)
	{
		$sql = "
			SELECT p.*
			FROM tbl_plantillas_cuentas AS pc
			JOIN cat_plantillas AS p
				ON p.id_plantilla = pc.id_plantilla
			WHERE pc.id_usuario = :id_usuario
				AND pc.id_tipo_comprobante = :id_tipo_comprobante
		";
		
		return $this->query($sql, array('id_usuario' => $id_usuario, 'id_tipo_comprobante' => $id_tipo_comprobante));
	}
	
	
	/**
	* Funcion que ejecuta el store procedure para insertar el registro
	* @return array
	*/	
	public function insertTemplate( $values ){
		
		return $this->executeSP( "stp_insPlantilla( :plantilla, 
												:descripcion, :activo )" , $values );
	}
	
	/**
	* Funcion que actualiza el estatus de un registro
	* @param array $values
	* @return array
	*/
	public function statusPlantilla( $values ){
		return $this->executeSP( "stp_estatusPlantilla( :id_plantilla , :activo)" , $values);
	}
	
	/**
	* Funcion que valida información
	* @param array $values
	* @return array
	*/
	public function validateData( $values ){
		
	}
}

?>