<?php

require_once ('Sucursal.php');

/**
 * @author Angel
 * @version 1.0
 * @created 20-ago-2013 02:57:18 p.m.
 */
class SucursalDAO extends Sucursal {

	/**
	 * Devuelve todos los registros de sucursales o solo el especificado por el id
	 * @param $data Valor entero para regresar un registro especifico. Si se omite, regresa todos los registros
	 * @return array
	 */
	public function listarSucursales($id_usuario = NULL) {
		return $this->executeSP("stp_listarSucursales(:id)", array('id' => $id_usuario));
	}
	
	/**
	 * Devuelve todos los registros de sucursales o solo el especificado por el id
	 * @param $data Valor entero para regresar un registro especifico. Si se omite, regresa todos los registros
	 * @return array
	 */
	public function listarSucursal($data = NULL, $id_usuario = NULL) {
		return $this->executeSP("stp_Sucursales(:id, :id_usuario)", array('id' => $data, 'id_usuario' => $id_usuario));
	}
	
	public function buildCombo( $idCliente = NULL){
		if(!is_int($idCliente) && $idCliente != NULL){
			$rows = $this->listarSucursal(NULL, $_SESSION['user']->id_usuario);
			$values['sNombre'] = $idCliente;
			$id_sucursal = $this->executeSP("stp_getIdSucursal(:sNombre)", $values);
			if(empty($id_sucursal[0]['id_sucursal'])){
				$id = 0;
			}else{
				$id = $id_sucursal[0]['id_sucursal'];
			}
		}else{
			$rows = $this->listarSucursal(null, $_SESSION['user']->id_usuario);
			$id = NULL;
		}
		
		$options = listForCombo2($rows, "id_sucursal", "nombre", $id);
		return $options;
	}
	/**
	 * Funcion que valida la informacion
	 * @param $data Array con la informacion capturada
	 * @return array
	 */
	public function validateData(array $data) {
		$data = (object) $data;
		$result = false;
		$errors = '';
		$ids = '';

		if (empty($data->nombre)) {
			$errors .= "- Nombre no puede estar en blanco.\n";
			$ids .= ',#nombre';
		}
		
		$bandera = true;
		if (empty($data->calle)) {
			$errors .= "- Calle y numero no puede estar en blanco.\n";
			$ids .= ',#calle';
		} else {
			$bandera = false;
		}
		if (empty($data->numero_ext)) {
			$errors .= "- Numero exterior no puede estar en blanco.\n";
			$ids .= ',#numero_ext';
		} else {
			$bandera = false;
		}
		
		if (empty($data->colonia)) {
			$errors .= "- Colonia no puede estar en blanco.\n";
			$ids .= ',#colonia';
		} else {
			$bandera = false;
		}
		
		if (empty($data->codigo_postal)) {
			$errors .= "- Codigo postal no puede estar en blanco.\n";
			$ids .= ',#codigo_postal';
		} else {
			$bandera = false;
		}
		
		if(!$bandera){
			$sucursal = $this->buscarSucursal(array(
				'nombre' => $data->nombre,
				'id_usuario' => $_SESSION['user']->id_usuario,
				'calle' => $data->calle,
				'codigo_postal' => $data->codigo_postal,
				'colonia' => $data->colonia,));
			if (!empty($sucursal['id_sucursal']) && $sucursal['id_sucursal'] != $data->id_sucursal) {
				$errors .= "- Nombre ya se encuentra registrado.\n";
				$ids .= ',#nombre';
			}
		}
		
		if (empty($data->id_estado)) {
			$errors .= "- Estado postal no puede estar en blanco.\n";
			$ids .= ',#id_estado';
		}
		
		if (empty($data->id_municipio)) {
			$errors .= "- Municipio postal no puede estar en blanco.\n";
			$ids .= ',#id_municipio';
		}

		$result['ok'] = empty($errors);
		$result['error']['ids'] = preg_replace('/^,(.*)/', '$1', $ids);
		$result['error']['msg'] = $errors;

		return array_utf8Decode($result);
	}

	/**
	 * Busca un registro con la coincidencia en la base de datos
	 * @param $data Array con el nombre a buscar y el id de usuario
	 * @return array
	 */
	public function buscarSucursal($data) {
		return array_shift($this->executeSP("stp_buscarSucursal(:nombre,:id_usuario,:calle_numero,:codigo_postal,:colonia)", $data));
	}

	/**
	 * Registra una nueva sucursal
	 * @param $data Array que contiene la informacion a registrar
	 * @return array
	 */
	public function insSucursal($data) {
		$sp ="stp_insSucursal(:id_usuario,
								:nombre,
								:calle,
								:numero_ext,
								:numero_int,
								:colonia,
								:codigo_postal,
								:id_estado,
								:id_municipio,
								:telefonos,
								:tasa_iva,
								:activo)";
		return array_shift($this->executeSP($sp, $data));		
	}

	/**
	 * Actualiza la sucursal especificada
	 * @param $data Array que contiene la informacion y Id del registro a actualizar
	 * @return array
	 */
	public function updSucursal($data) {
		$sp = "stp_updSucursal(:id_sucursal,
				 :id_usuario, 
				 :nombre,
				 :calle,
				 :numero_ext,
				 :numero_int,
				 :colonia,
				 :codigo_postal,
				 :id_estado,
				 :id_municipio,
				 :telefonos,
				 :tasa_iva,
				 :activo)";
		return array_shift($this->executeSP($sp, $data));
	}

	/**
	 * Cambia el estatus de activo del registro especificado
	 * @param $id_sucursal Valor entero, Id del registro
	 * @return array
	 */
	public function statusSucursal($id_sucursal) {
		$this->executeSP("stp_statusSucursal(:id_sucursal)", array('id_sucursal' => $id_sucursal));
	}
	
	public function getConfiguracionCuenta($id_usuario) {
		return $this->executeSP("stp_ConfiguracionCuenta(:id_usuario)", array('id_usuario' => $id_usuario));
	}
	
	public function getInfoSucursalMatriz($id_usuario) {
		return $this->executeSP("stp_SucursalMatriz(:nId_usuario)", array('nId_usuario' => $id_usuario));
	}
	
	public function recuperarSucursal($id_sucursal, $id_usuario = NULL) {
		$sql = "stp_recuperarSucursal(:id_sucursal, :id_usuario)";
		return array_shift($this->executeSP($sql, array('id_sucursal' => $id_sucursal, 'id_usuario' => $id_usuario)));
	}

}

?>