<?php
require_once ('UnidadMedida.php');

/**
 * @author Angel
 * @version 1.0
 * @created 05-ago-2013 05:12:55 p.m.
 */
class UnidadMedidaDAO extends UnidadMedida
{

	/**
	 * Devuelve todos los registros de unidad de medida o solo el especificado por el id
	 * @param $data Valor entero para regresar un registro especifico. Si se omite, regresa todos los registros
	 * @return array
	 */
	public function listarUnidadMedida($data = null) {
		return $this->executeSP("stp_UnidadesMedida(:id)", array('id' => $data));
	}
	
	public function buildCombo($id_unidad = null){
		$rows = $this->listarUnidadMedida($id_unidad);
		$options = listForCombo($rows, "id_unidad_medida", "nombre");
		return $options;
	}
	/**
	 * Funcion que valida la informacion
	 * @param $data Array con la informacion capturada
	 * @return array
	 */
	public function validateData($data){
		$data = (object) $data;
		$result = false;
		$errors = '';
		$ids = '';

		if (empty($data->nombre)) {
			$errors .= "- Nombre no puede estar en blanco.\n";
			$ids .= ',#nombre';
		} else {
			$unidad = $this->buscarUnidadMedida(array('nombre' => $data->nombre));
			if (! empty($unidad['id_unidad_medida']) && $unidad['id_unidad_medida'] != $data->id_unidad) {
				$errors .= "- Nombre ya se encuentra registrado.\n";
				$ids .= ',#nombre';
			}
		}

		if (empty($data->descripcion)) {
			$errors .= "- Descripcion no puede estar en blanco.\n";
			$ids .= ',#descripcion';
		}

		$result['ok'] = empty($errors);
		$result['error']['ids'] = preg_replace('/^,(.*)/','$1',$ids);
		$result['error']['msg'] = $errors;

		return array_utf8Decode($result);
	}

	/**
	 * Busca un registro con la coincidencia en la base de datos
	 * @param $data Nombre a buscar
	 * @return array
	 */
	public function buscarUnidadMedida($data){
		return array_shift($this->executeSP('stp_buscarUnidadMedida(:nombre)', $data));
	}

	/**
	 * Registra una nueva unidad de medida
	 * @param $data Array que contiene la informacion a registrar
	 * @return array
	 */
	public function insertUnidadMedida($data) {
		$valida = $this->executeSP('stp_insUnidadMedida(:nombre, :descripcion, :activo)', $data);
		return array_shift($valida);
	}

	/**
	 * Actualiza la unidad de medida especificada
	 * @param $data Array que contiene la informacion y Id del registro a actualizar
	 * @return array
	 */
	public function updateUnidadMedida($data) {
		$valida = $this->executeSP("stp_updUnidadMedida(:id_unidad, :nombre, :descripcion, :activo)", $data);
		return array_shift($valida);
	}

	/**
	 * Cambia el estatus de activo del registro especificado
	 * @param $id_unidad_medida Valor entero, Id del registro
	 * @return array
	 */
	public function statusUnidadMedida($id_unidad_medida){
		return $this->executeSP("stp_statusUnidadMedida(:id_unidad)", array('id_unidad' => $id_unidad_medida));
	}

}
?>