<?php

/**
 * Clase DAO que usa los objetos creados de la clase que hereda
 * @author Eliud Trejo
 */

require FS_CLASSES . "Usuarios.php";

class UsuariosDAO extends Usuarios{
	
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con la lista de todos elementos
	* @return array
	*/
	public function usuariosPorTipo($tipo)
	{
		return $this->executeSP('stp_Usuarios(:tipo)', array('tipo' => $tipo));
	}
	
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con la lista de todos elementos
	* @return array
	*/
	public function listarUsuarios()
	{
		return $this->executeSP('stp_cargaUsuarios(NULL)');
	}
	
	/**
	* Funcion que retorna la información del usuario
	* @param $id_user Id del usuario del cual se requiere la información.
	*/
	public function getUserInfo($id_user = null)
	{
		if (is_null($id_user)) $id_user = $this->getId_user();
		$row = array();
		$row = $this->executeSP('stp_cargaUsuarios(:id_usuario)', array('id_usuario' => $id_user));
		$row = array_shift($row);
		
		$this->setId_user($row['id_usuario']);
		$this->setStatus($row['activo']);
		
		return $row;
	}

	/**
	* Funcion que registra las sesión
	*/
	public function registrarSesion()
	{
		$sql = "stp_registraSesion(:id_user)";
		return $this->executeSP($sql, array('id_user' => $this->getId_user()));
	}
	
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con
	 * la información del usuario especificado.
	 * @param $id id del usuario a buscar
	* @return array
	*/
	public function buscarUsuario($id)
	{
		$row = array();
		$row = $this->executeSP('stp_cargaUsuarios(:id_usuario)', array('id_usuario' => $id));
		$row = array_shift($row);

		return $row;
	}

	/**
	* Funcion que ejecuta el store procedure devolviendo un array con
	 * la información del usuario especificado.
	 * @param $username Nombre del usuario a buscar
	* @return array
	*/
	public function findUser($username)
	{
		$row = array();
		$row = $this->executeSP('stp_BuscaUsuario(:usuario)', array('usuario' => $username));
		$row = array_shift($row);

		return $row;
	}

	/**
	* Funcion que busca al usuario por su email
	 * @param $email Email del usuario a buscar
	* @return array
	*/
	public function findUserByEmail($email)
	{
		$row = array();
		$sql = "
			SELECT * FROM cat_usuarios
			WHERE e_mail = '". $email ."';
		";
		$row = $this->query( $sql , array( 'e_mail' => $email ) );
		$row = array_shift($row);

		return $row;
	}
	
	/**
	* Funcion que ejecuta el store procedure para actualizar la información del usuario
	 * @param $data datos del usuario a actualizar
	* @return array
	*/
	public function updateUser($data)
	{
		$result = false;
		$sql = "stp_modificaUsuario(:id_usuario, :usuario, :nombre, 
									:contrasenia, :activo, :permisos, :e_mail)";

		return $this->executeSP($sql, $data);
	}

	/**
	* Funcion que ejecuta el store procedure para insertar la información del usuario
	 * @param $data datos del usuario a insertar
	* @return array
	*/
	public function insertUser($data)
	{
		$result = false;
		$sql = "stp_registraUsuario(:usuario, :nombre, :contrasenia, :tipo_usuario, :activo, :permisos, :e_mail)";

		return array_shift($this->executeSP($sql, $data));
	}

	/**
	* Funcion que ejecuta el store procedure para retornar 
	* el tipo de perfil del usuario especificado
	* @param $id_user id del usuario del usuario cuyo tipo de perfil se quiere recuperar
	* @return array
	*/
	public function listarPerfiles($id_user = null)
	{
		if (is_null($id_user)) $id_user = $this->getId_user();

		$row = array();
		$sql = "stp_PerfilesUsuarios(:id_user)";
		$perfiles = $this->executeSP($sql, array('id_user' => $id_user));

		foreach($perfiles as $perfil) $row[] = $perfil['id_perfil'];
		
		return $row;
	}
	
	/**
	* Funcion que actualiza el estatus de un usuario
	* @param $data datos del usuario cuyo estatus sera cambiado
	* @return array
	*/
	public function statusUser( $data ){
		$sql = "stp_registraStatusUsuario(:id_user, :activo)";
		
		return $this->executeSP($sql, $data);
	}
	
	/**
	* Funcion que valida la información de un usuario
	* @param $data datos del usuario cuyo estatus sera cambiado
	* @return array
	*/
	public function validateData($data)
	{
		$permisos = $data['permisos'];
		$data = (object) $data;
		$result = false;
		$errors = '';
		$ids = '';

		if (empty($data->usuario)) {
			$errors .= "- Clave de usuario no puede estar en blanco.\n";
			$ids .= ',#usuario';
		} else {
			$usr = $this->findUser($data->usuario);
			if (! empty($usr['id_user']) && $usr['id_user'] != $data->id_user) {
				$errors .= "- Clave de usuario ya se encuentra registrada.\n";
				$ids .= ',#usuario';
			}
		}

		if (empty($data->nombre)) {
			$errors .= "- Nombre del usuario no puede estar en blanco.\n";
			$ids .= ',#nombre';
		}

		if (empty($permisos)) {
			$errors .= "- Debe indicar al menos un perfil de permisos.\n";
			$ids .= ',#perfil';
		}

		$result['ok'] = empty($errors);
		$result['error']['ids'] = preg_replace('/^,(.*)/','$1',$ids);
		$result['error']['msg'] = $errors;

		return array_utf8Decode($result);
	}
	
	public function recuperarUsuario($id_usuario){
		$sql = "stp_recuperaInfoUsuario(:id_usuario)";
		return array_shift($this->executeSP($sql, array('id_usuario' => $id_usuario)));
	}
}

?>