<?php
/**************************************************************************************************/
/***                    Archivo para funciones de PHP globales del framework                    ***/
/**************************************************************************************************/

/**
 * Muestra el contenido de una variable y detiene la
 * ejecución para facilitar la depuración de procesos.
 * @param mixed $var
 * @param boolean $halt default = true
 * @return void
 */
function debug($var, $halt = true)
{
	echo '<pre>';
	print_r($var);

	if ($halt) exit();
}

function initSession()
{
	session_save_path (FS_SESSIONS);

	if (! file_exists(FS_SESSIONS)) {
		mkdir(FS_SESSIONS , 0777);
	}

	if (! session_id()) {
		session_start();
	}
}

function cleanupSessions($hrs = 24)
{
	if (file_exists(FS_SESSIONS)) {
		$_Dir = dir(FS_SESSIONS);
		$_curTime = time();

		while (false !== ($_sessFile = $_Dir->read())) {
			if (is_file(FS_SESSIONS . $_sessFile)) {
				$_modTime = filemtime(FS_SESSIONS . $_sessFile);
				$_hrsDiff = ($_curTime - $_modTime) / 3600;
				if ($_hrsDiff > $hrs) unlink(FS_SESSIONS . $_sessFile);
			}
		}

		$_Dir->close();
	}
}

/**
 * Genera un cadena de autenticidad para comprobación de las sesión de trabajo.
 * @param object $object
 * @return string
 */
function sessionHash($object)
{
	return md5(json_encode($object));
}

/**
 * Devuelve un string codificado de un arreglo para el envio de parámetros a módulos
 * @param array $params
 * @return string
 */
function encodeQueryString($params)
{
	$params['rnd'] = rand(0, 65535);
	$_tmp = json_encode($params);
	$_tmp = str_rot13($_tmp);
	$_tmp = strrev($_tmp);
	$_tmp = base64_encode($_tmp);

	return '?qs=' . $_tmp;
}

/**
 * Devuelve un objecto de parametros en base a un query string codificado
 * @param string $get
 * @return object
 */
function decodeQueryString($get)
{
	$_tmp = base64_decode($get);
	$_tmp = strrev($_tmp);
	$_tmp = str_rot13($_tmp);
	$_tmp = json_decode($_tmp);

	return $_tmp;
}

/**
 * Devuelve el nombre del mes por el numero indicado
 * @param integer $nMonth
 * @param integer $len
 * @return string
 */
function monthName($nMonth, $len = null)
{
	GLOBAL $aMonths;
	$_result = $aMonths[$nMonth - 1];
	return substr($_result, 0, is_numeric($len) ? $len : strlen($_result));
}

/**
 * Convierte una fecha en formato ISO una fecha con el nombre corto dd/Mmm/aaaa
 * @param date $date fecha en formato aaaa-mm-dd
 * @return string fecha en formato dd/Mmm/aaaa
 */
function shortDate($date)
{
	if (! preg_match('/^\d{4}.\d{2}.\d{2}.*/', $date)) {
		return $date;
	}
	
	$_month = monthName(substr($date, 5, 2), 3);
	return preg_replace('/^(\d{4}).(\d{2}).(\d{2})(.*)/', '$3/' . $_month . '/$1$4', $date);
}

function array_utf8($arreglo)
{
	//-- Barrido del arreglo por llaves para codificar los elementos.
	foreach ($arreglo as $key => $value) {
		//-- Si elemento es otro arreglo, realiza llamada recursiva.
		if (is_array($value)) {
			$arreglo[$key] = array_utf8($value);
		} else {
			$arreglo[$key] = utf8_encode($value);
		}
	}

	return $arreglo;
}

function json_encode_utf8($arreglo)
{
	return json_encode(array_utf8($arreglo));
}

function array_utf8Decode($arreglo)
{
//-- Barrido del arreglo por llaves para codificar los elementos.
	foreach ($arreglo as $key => $value) {
		//-- Si elemento es otro arreglo, realiza llamada recursiva.
		if (is_array($value)) {
			$arreglo[$key] = array_utf8Decode($value);
		} else {
			$arreglo[$key] = utf8_decode($value);
		}
	}

	return ($arreglo);
}

function isNaN($var)
{
	return preg_match('/^[-]?[0-9]+([\.][0-9]+)?$/', $var);
}

/**
 * Convierte una fecha del formato dd/mm/aaaa al formato ISO
 * @param date $date fecha en formato dd/mm/aaaa
 * @return string
 */
function dateISO(&$date)
{
	if (empty($date)) {
		$date = null;
	} else {
		if(! preg_match('/^(\d{4})(\/|-)(\d{2})(\/|-)(\d{2})/', $date)) {
			$date = preg_replace('/(\d{2}).(\d{2}).(\d{4})/', '$3-$2-$1', $date);
		}
	}
}

/**
 * Convierte una fecha del formato ISO al formato dd/mm/aaaa
 * @param date $date fecha en formato aaaa-mm-dd
 * @return string
 */
function dateMX(&$date)
{
	if (empty($date)) {
		$date = null;
	} else {
		if(preg_match('/^(\d{4})(\/|-)(\d{2})(\/|-)(\d{2})/', $date)) {
			$date = preg_replace('/(\d{4})-(\d{2})-(\d{2})/', '$3/$2/$1', $date);
		}
	}
	return($date);
}

//-- Limpia formato de moneda de un numero
function cleanMoney(&$value)
{
	if (trim($value) == '') {
		$value = null;
	} else {
		$value = preg_replace('/[\$,]/', '', $value);
	}
}

//-- Limpia formato de porcentaje de un numero
function cleanPercent(&$value)
{
	if (trim($value) == '') {
		$value = null;
	} else {
		$value = preg_replace('/[\%,]/', '', $value);
	}
}

//-- Convierte una variable en blanco a null para envio a Base de datos
function blankToNull(&$value)
{
	if (trim($value) == '') {
		$value = null;
	}
}

function pesos($variable, $decimales = 2, $blank = FALSE, $str_blank = "") {
//-- limpiar caracteres que no sean digitos o puntos.
	$negativos = substr($variable, 0, 1);
	if($negativos != '-') $negativos = '';
	$variable = preg_replace("/[^\d|\.]/", "", $variable);

	$pesos = $negativos."$" . number_format($variable, $decimales, '.', ',');

	return (($blank && floatval($variable) == 0) ? $str_blank : $pesos);
}

/**
 * Devuelve una fecha con un determinado desplazamiento en meses hacia adelante o hacia atras
 * @author Carlos Almaguer
 * @param string $tcFecha Fecha a desplazar en formato aaaa/mm/dd, aaaa-mm-dd, aaaa.mm.dd
 * @param integer $nMonths Cantidad de meses a desplazar, positivo hacia adelante, negativo hacia atrás [opcional, default 1 mes hacia adelante]
 * @param integer $nDays Cantidad de dias a desplazar, positivo hacia adelante, negativo hacia atrás [opcional, default 0 días]
 * @return string Fecha desplazada en formato aaaa/mm/dd con el separador indicado
 */
function goMonth($tcFecha, $nMonths = 1, $nDays = 0, $separator = "-") {
	$tcFecha = preg_replace('/\x20\d{2}:\d{2}:\d{2}/', '', $tcFecha);

	//-- Separa la cadena contenindo la fecha
	if (!preg_match('/^\d{4}.\d{2}.\d{2}/', $tcFecha)) {
		dateISO($tcFecha);
	}
	
	list($nAnio, $nMes, $nDia) = preg_split("/\/|-|\./", $tcFecha);

	//-- Genera timestamp con el desplazamiento de los meses
	$fecha = mktime(12, 0, 0, $nMes + $nMonths, $nDia + $nDays, $nAnio);

    return date("Y" . $separator . "m" . $separator . "d", $fecha);
}

/**
 * Leading zeros
 *
 * Formatea con ceros a la izquierda un numero hasta una longitud determinada
 *
 * @author Carlos Almaguer
 *
 * @param integer $valor
 * @param integer $longitud
 * @return string
 */
function leadZero($valor, $longitud , $dir = STR_PAD_LEFT)
{
	return (STR_PAD($valor, $longitud, "0", $dir));
}


/**
 * Conversion de cantidades numericas a texto
 *
 * Convierte un número entero de hasta 11 posiciones a su representación en texto
 *
 * @author Carlos Almaguer
 *
 * @param integer $tnNumero cantidad a convertir (máximo 11 digitos)
 * @return string
 */
function numeroATexto ($tnNumero)
{
	$aCientos = array("CIENTO",
			"DOSCIENTOS",
			"TRESCIENTOS",
			"CUATROCIENTOS",
			"QUINIENTOS",
			"SEISCIENTOS",
			"SETECIENTOS",
			"OCHOCIENTOS",
			"NOVECIENTOS"
	);

	$aDecenas = array(
			"UN",
			"DOS",
			"TRES",
			"CUATRO",
			"CINCO",
			"SEIS",
			"SIETE",
			"OCHO",
			"NUEVE",
			"DIEZ",
			"ONCE",
			"DOCE",
			"TRECE",
			"CATORCE",
			"QUINCE",
			"DIECISEIS",
			"DIECISIETE",
			"DIECIOCHO",
			"DIECINUEVE",
			"VEINTE",
			"VEINTIUN",
			"VEINTIDOS",
			"VEINTITRES",
			"VEINTICUATRO",
			"VEINTICINCO",
			"VEINTISEIS",
			"VEINTISIETE",
			"VEINTIOCHO",
			"VEINTINUEVE",
			"TREINTA",
			"TREINTA Y UN",
			"TREINTA Y DOS",
			"TREINTA Y TRES",
			"TREINTA Y CUATRO",
			"TREINTA Y CINCO",
			"TREINTA Y SEIS",
			"TREINTA Y SIETE",
			"TREINTA Y OCHO",
			"TREINTA Y NUEVE",
			"CUARENTA",
			"CUARENTA Y UN",
			"CUARENTA Y DOS",
			"CUARENTA Y TRES",
			"CUARENTA Y CUATRO",
			"CUARENTA Y CINCO",
			"CUARENTA Y SEIS",
			"CUARENTA Y SIETE",
			"CUARENTA Y OCHO",
			"CUARENTA Y NUEVE",
			"CINCUENTA",
			"CINCUENTA Y UN",
			"CINCUENTA Y DOS",
			"CINCUENTA Y TRES",
			"CINCUENTA Y CUATRO",
			"CINCUENTA Y CINCO",
			"CINCUENTA Y SEIS",
			"CINCUENTA Y SIETE",
			"CINCUENTA Y OCHO",
			"CINCUENTA Y NUEVE",
			"SESENTA",
			"SESENTA Y UN",
			"SESENTA Y DOS",
			"SESENTA Y TRES",
			"SESENTA Y CUATRO",
			"SESENTA Y CINCO",
			"SESENTA Y SEIS",
			"SESENTA Y SIETE",
			"SESENTA Y OCHO",
			"SESENTA Y NUEVE",
			"SETENTA",
			"SETENTA Y UN",
			"SETENTA Y DOS",
			"SETENTA Y TRES",
			"SETENTA Y CUATRO",
			"SETENTA Y CINCO",
			"SETENTA Y SEIS",
			"SETENTA Y SIETE",
			"SETENTA Y OCHO",
			"SETENTA Y NUEVE",
			"OCHENTA",
			"OCHENTA Y UN",
			"OCHENTA Y DOS",
			"OCHENTA Y TRES",
			"OCHENTA Y CUATRO",
			"OCHENTA Y CINCO",
			"OCHENTA Y SEIS",
			"OCHENTA Y SIETE",
			"OCHENTA Y OCHO",
			"OCHENTA Y NUEVE",
			"NOVENTA",
			"NOVENTA Y UN",
			"NOVENTA Y DOS",
			"NOVENTA Y TRES",
			"NOVENTA Y CUATRO",
			"NOVENTA Y CINCO",
			"NOVENTA Y SEIS",
			"NOVENTA Y SIETE",
			"NOVENTA Y OCHO",
			"NOVENTA Y NUEVE"
	);

	$lcTexto = "";
	list($lcNumero, $lcDecimales) = explode("\.", $tnNumero);
	$lcNumero = leadZero($lcNumero, 11);

	do {
		//*------------------------------*
		//* Determina miles de millones. *
		//*------------------------------*
		$w_mimi = false;

		if(substr($lcNumero, 0, 2) <> "00") {
			$lcTexto = $aDecenas[intval(substr($lcNumero, 0, 2)) - 1] . " MIL";

			if(substr($lcNumero, 2, 9) == "000000000") {
				$lcTexto .= " MILLONES DE";
				break;
			}

			$w_mimi = true;
		}

		//*--------------------------------*
		//* Determina cientos de millones. *
		//*--------------------------------*
		$w_cimi = false;

		if(substr($lcNumero, 2, 1) <> "0") {
			if(substr($lcNumero, 2, 3) == "100") $lcTexto .= " CIEN";
			else $lcTexto .= " " . $aCientos[intval(substr($lcNumero, 2, 1)) - 1];

			if(substr($lcNumero, 3, 8) == "00000000") {
				$lcTexto .= " MILLONES DE";
				break;
			}

			$w_cimi = true;
		}

		//*--------------------------------*
		//* Determina decenas de millones. *
		//*--------------------------------*
		$w_dimi = false;

		if(substr($lcNumero, 3, 2) <> "00") {
			if(substr($lcNumero, 0, 3) == "000" && substr($lcNumero, 3, 2) == "01")
				$lcTexto .= " " . $aDecenas[intval(substr($lcNumero, 3, 2)) - 1] . " MILLON";
			else $lcTexto .= " " . $aDecenas[intval(substr($lcNumero, 3, 2)) - 1] . " MILLONES";

			if(substr($lcNumero, 5, 6) == "000000") {
				$lcTexto .= " DE";
				break;
			}

			$w_dimi = true;
		}
		elseif($w_mimi || $w_cimi) $lcTexto .= " MILLONES";

		//*-----------------------------*
		//* Determina cientos de miles. *
		//*-----------------------------*
		$w_cemi = false;

		if(substr($lcNumero, 5, 1) <> "0") {
			if(substr($lcNumero, 5, 3) == "100") {
				$lcTexto .= " CIEN";

				if(substr($lcNumero, 8, 3) == "000") {
					$lcTexto .= " MIL";
					break;
				};
			}
			else $lcTexto .= " " . $aCientos[intval(substr($lcNumero, 5, 1)) - 1];

			$w_cemi = true;
		};

		//*-----------------------------*
		//* Determina decenas de miles. *
		//*-----------------------------*
		$w_demi = false;

		if(substr($lcNumero, 6, 2) <> "00") {
			if(substr($lcNumero, 0, 6) == "000000" && substr($lcNumero, 6, 2) == "01") $lcTexto .= " UN MIL";
			else $lcTexto .= " " . $aDecenas[intval(substr($lcNumero, 6, 2)) - 1] . " MIL";

			$w_demi = true;
		}
		else {
			if($w_cemi) $lcTexto .= " MIL";
		};

		//*--------------------*
		//* Determina cientos. *
		//*--------------------*
		$w_cipe = false;

		if(substr($lcNumero, 8, 1) <> "0") {
			if(substr($lcNumero, 8, 3) == "100") {
				$lcTexto .= " CIEN";
				break;
			}
			else $lcTexto .= " " . $aCientos[intval(substr($lcNumero, 8, 1)) - 1];

			$w_cipe = true;
		};

		//*--------------------*
		//* Determina decenas. *
		//*--------------------*
		if(substr($lcNumero, 9, 2) <> "00")
			$lcTexto .= " " . $aDecenas[intval(substr($lcNumero, 9, 2)) - 1];
	} while (0);

	if (empty($lcTexto)) $lcTexto = "CERO";

	return $lcTexto;
}

/**
 * Conversion de cantidades monetarias a texto
 *
 * Convierte una cifra monetaria de hasta 11 enteros y 2 decimales a su representación textual
 *
 * @author Carlos Almaguer
 *
 * @param decimal(11, 2) $tnNumero cantidad a convertir
 * @param string $tcMoneda nombre de la moneda, default = "PESOS"
 * @param string $tcDenominacion denominación de la moneda, default = "M.N."
 * @return string
 */
function monedaATexto($tnNumero, $tcMoneda = "PESOS", $tcDenominacion = "M.N.")
{
	list($lcNumero, $lcCents) = explode(".", $tnNumero);
	$lcNumero = leadZero($lcNumero, 11);

	$lcTexto = numeroATexto($lcNumero);

	$lcTexto = trim($lcTexto)
			. (! empty($tcMoneda) ? " " . $tcMoneda : "")
			. " " . leadZero($lcCents, 2 , STR_PAD_RIGHT) . "/100"
			. (! empty($tcDenominacion) ? " " . $tcDenominacion : "");

	return ($lcTexto);
}

/**
 * Convierte una fecha de formaro ISO a UNIX_TIMESTAMP.
 * @author calmaguer@softnet.com.mx
 * @param DATE $iso_date Fecha con formato ISO.
 * @return timestamp UNIX_TIMESTAMP.
 */
function ISO2Unix( $iso_date )
{	
	$time = null;

	if (! empty($iso_date)) {
		$datec = explode(' ', $iso_date);
		list($y, $mo, $d) = preg_split("/\/|-|\./", $datec[0]);

		// Separa cadena para la hora
		if (! preg_match("/\d{2}:\d{2}:\d{2}/", $datec[1])) $datec[1] = '12:00:00';
		list($h, $m, $s) = preg_split("/:/", $datec[1]);

		$time = mktime( $h , $m , $s , $mo , $d , $y );
	}
	
	return $time;
}

 /**
  * Funcion que carga el grid con los datos obtenidos de un array
  * 
  * @param type $rows Array con la información a procesar.
  * @param type $wStyle	Valida si es necesario añadir stilo al grid que se retornara
  * @param type $paramToStyle Es el parametro por el cual decidira, si se añadira o no stilo
  * @param string $paramToDate Es el parametro que contiene el nombre del campo a poner en formato de fecha
  * @param string $paramToMoney Es el parametro que contiene el nombre del campo a poner en formato de moneda
  * @param bool $keepID Indica si la columna de id de los registro del grid se conserva o elimina (default se coserva)
  * @return array
  */
	function dataToGrid( $rows , $wStyle = null , $paramToStyle = null , 
								$paramToDate = null , $paramToMoney = null,
								$keepID = true, $userdata = 0){
		$grid = array();
		$grid['rows'] = array();

			if( $wStyle ){
				foreach ($rows as $row) {
					if ($paramToDate) {
						$dfs = explode(',', $paramToDate);
						
						foreach ($dfs AS $key) {
							dateMX($row[$key]);
						}
					}
					
					if ($paramToMoney) {
						$dfs = explode(',', $paramToMoney);
						
						foreach ($dfs AS $key) {
							$row[$key] = pesos($row[$key]);
						}
					}
					
					$style = $row[$paramToStyle] ? '' : DISABLED_ROW_STYLE;
					
					if (! $keepID) {
						$id = array_shift($row);
					} else {
						$id = reset($row);
					}
					
					if ($userdata > 0) {
						$udata = array_splice($row, -$userdata);
					}
					
					$grid['rows'][] = array(
						'id' => $id,
						'data' => array_values($row),
						'style' => $style,
						'userdata' => $udata
					);
				}
			} else {
				$grid['rows'] = array();
				//-- Armado de estructura JSON para el grid
				foreach ($rows as $row) {
					if ($paramToDate) {
						$dfs = explode(',', $paramToDate);
						
						foreach ($dfs AS $key) {
							dateMX($row[$key]);
						}
					}
					
					if ($paramToMoney) {
						$dfs = explode(',', $paramToMoney);
						
						foreach ($dfs AS $key) {
							$row[$key] = pesos($row[$key]);
						}
					}

					if (! $keepID) {
						$id = array_shift($row);
					} else {
						$id = reset($row);
					}
					
					$grid['rows'][] = array(
						'id' => $id,
						'data' => array_values($row)
					);
				}

			}

		return $grid;
	}
	
	/**
	* Funcion que ejecuta el store procedure para obtener todos los datos y
	* devuelve el listado de todos los elementos para armado de combo
	* @return string 
	*/		
	function listForCombo($rows, $idName, $nameField, $id = null){
		//Cargar los objetos para desplegarlos al select
		$option = '<option value="0">Seleccione una opci&oacute;n</option>';
		foreach( $rows as $row ){
			if ($row[$idName] == $id) $selected = " selected=\"selected\"";
			$option.='<option value="' . 
					$row[$idName] . '"' . 
					$selected . '>' .
					$row[$nameField] . '</option>';
			$selected='';
		}
		return $option;
	}
	
	/**
	* Funcion que ejecuta el store procedure para obtener todos los datos y
	* devuelve el listado de todos los elementos para armado de combo
	* NO Devuelve el option con value = 0 -----------------
	* @return string 
	*/		
	function listForCombo2($rows, $idName, $nameField, $id = null){
		//Cargar los objetos para desplegarlos al select
		foreach( $rows as $row ){
			if ($row[$idName] == $id) $selected = " selected=\"selected\"";
			$option.='<option value="' . 
					$row[$idName] . '"' . 
					$selected . '>' .
					$row[$nameField] . '</option>';
			$selected='';
		}
		return $option;
	}
	
	/**
	 * Convierte un arreglo de opciones a una lista
	 * @param array $array Arreglo con lista de opciones a generar
	 * @param mixed $default Valor de la opcion a seleccionar por default
	 * @param bool $byKey Define si el valor de la opcion es en base a la llave del arreglo
	 * @return string
	 */	
	function arrayToOptionTags($array, $default = null, $byKey = false)
	{
		$options = '';

		if (count($array) > 0) {
			foreach ($array as $key => $texto) {
				$value = ($byKey ? $key : $texto);
				$options .= '<option value="' . $value . '"'
					. ($value == $default ? ' selected="selected"' : '')
					. '>' . $texto . '</option>';
			}
		}

		return $options;
	}
	
	
	/**
	 * Función que importa un archivo (función prototipo)
	 * @param type $sourceName Nombre del archivo a importar
	 * @param type $sourcePath Ruta origen del archivo
	 * @param type $destinationDirectory Ruta origen del archivo
	 * @param type $destinationFileName Nombre que recibira el archivo a importar 
	 *				si se quiere cambiar
	 * @param type $idFile
	 * @author Eliud Trejo
	 */
	function importFile( $sourceName = null, $sourcePath = null , $destinationDirectory = null , $destinationFileName = null , $idFile = null ){
//			//vemos si la carpeta destino existe si no, se crea
//			if( !file_exists($destinationDirectory) ){
//				mkdir($destinationDirectory,0777);
//			}
//			
//			if(file_exists($destinationDirectory)){
//				copy($sourcePath.$sourceName, $destinationDirectory.$destinationFileName);
//			}
	}

	/**
	* Almacena información como un archivo con el Byte Order Mark (BOM) para UTF8
	* @param string $filename Nombre del archivo
	* @param string $contents Información a guardar
	*/
   function saveUTF8File($filename, $contents)
   {
	   $dhandle = fopen($filename, "w");
	   fwrite($dhandle, pack("CCC", 0xef, 0xbb, 0xbf));
	   fwrite($dhandle, utf8_encode($contents));
	   fclose($dhandle);
   }
?>